﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;

namespace BlockPuzzleSolution
{
    public class Block
    {
        public const int unitLength = 10;

        public string HtmlColor;
        public int[] Matrix = new int[25];
        public Point Position;
        public Size BlockSize;

        [XmlIgnore()]
        public Color MyColor
        {
            set { HtmlColor = ColorTranslator.ToHtml(value); }
            get { return ColorTranslator.FromHtml(HtmlColor); }
        }

        public int readMatrix(int x, int y)
        {
            return Matrix[y * 5 + x];
        }
        public void setMatrix(int x, int y, int v)
        {
            Matrix[y * 5 + x] = v;
        }

        public Block()
        {
            MyColor = Color.Red;
            for (int i = 0; i < 5; i++) for (int j = 0; j < 5; j++) setMatrix(i, j, 0);
            Position = new Point(-1, -1);
            BlockSize = new Size(0, 0);
        }

        public void Rotate()
        {
            int[] newMatrix = new int[25];
            Size newSize = new Size();
            newSize.Width = BlockSize.Height;
            newSize.Height = BlockSize.Width;
            for (int i = 0; i < 25; i++) newMatrix[i] = 0;
            for (int i = 0; i < BlockSize.Width; i++) for (int j = 0; j < BlockSize.Height; j++)
                {
                    newMatrix[(BlockSize.Width - i - 1) * 5 + j] = readMatrix(i, j);
                }
            BlockSize = newSize;
            Matrix = newMatrix;
        }


        public Image GetImage()
        {
            Bitmap bBitMap = new Bitmap(5 * unitLength, 5 * unitLength);
            Graphics objGraphics = Graphics.FromImage(bBitMap);
            Bitmap bTexture = new Bitmap(unitLength, unitLength);
            Graphics graTexture = Graphics.FromImage(bTexture);
            LinearGradientBrush brTexture = new LinearGradientBrush(new Rectangle(0, 0, unitLength, unitLength), Color.White, this.MyColor, LinearGradientMode.ForwardDiagonal);
            graTexture.FillRectangle(brTexture, 0, 0, unitLength, unitLength);
            graTexture.Flush();
            TextureBrush br = new TextureBrush(bTexture);
            Pen wPen = new Pen(Color.Black, 1);
            for (int i=0; i<BlockSize.Width; i++)
                for (int j = 0; j < BlockSize.Height; j++)
                {
                    if (readMatrix(i, j) == 1)
                    {
                        objGraphics.FillRectangle(br, i*unitLength, (4-j)*unitLength, unitLength, unitLength);
                        // draw border
                        //left
                        if (i == 0 || readMatrix(i - 1, j) == 0)
                        {
                            objGraphics.DrawLine(wPen, i * unitLength, (4 - j) * unitLength, i * unitLength, (5 - j) * unitLength-1);
                        }
                        //right
                        if (i == (BlockSize.Width - 1) || readMatrix(i + 1, j) == 0)
                        {
                            objGraphics.DrawLine(wPen, (i + 1) * unitLength-1 , (4 - j) * unitLength, (i + 1) * unitLength-1 , (5 - j) * unitLength-1);
                        }
                        //top
                        if (j == (BlockSize.Height - 1) || readMatrix(i, j + 1) == 0)
                        {
                            objGraphics.DrawLine(wPen, i * unitLength, (4 - j) * unitLength, (i + 1) * unitLength-1, (4 - j) * unitLength);
                        }
                        //bottom
                        if (j == 0 || readMatrix(i, j - 1) == 0)
                        {
                            objGraphics.DrawLine(wPen, i * unitLength, (5 - j) * unitLength-1 , (i + 1) * unitLength-1, (5 - j) * unitLength-1 );
                        }
                    }
                }
            objGraphics.Flush();
            return bBitMap;
        }

    }
}
