﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml.Serialization;

namespace BlockPuzzleSolution
{
    public partial class Form1 : Form
    {
        Puzzle Puzzle;
        string FileName;
        bool isChanged;
        private List<Color> preferredColors = new List<Color>();
        public Form1()
        {
            InitializeComponent();
            Puzzle = new Puzzle();
            Puzzle.initPuzzle(5, 5);
            lblPuzzle.Text = "Width:" + this.Puzzle.PuzzleSize.Width.ToString() +
                "    Height:" + this.Puzzle.PuzzleSize.Height.ToString();
            show_blocks();
            setFileName("NewPuzzle.xml");
            isChanged = false;
            preferredColors.Add(Color.Red);
            preferredColors.Add(Color.Yellow);
            preferredColors.Add(Color.Blue);
            preferredColors.Add(Color.MediumSlateBlue);
            preferredColors.Add(Color.OrangeRed);
            preferredColors.Add(Color.Gold);
            preferredColors.Add(Color.Green);
            preferredColors.Add(Color.DodgerBlue);
            preferredColors.Add(Color.Lime);
        }

        private void btnNewBlock_Click(object sender, EventArgs e)
        {
            //find preferred color
            Color pc = Color.Red;
            foreach (Color c in preferredColors)
            {
                bool colorExists = false;
                foreach (Block b in Puzzle.Blocks)
                {
                    if (b.MyColor == c) { colorExists = true; break; }
                }
                if (!colorExists) { pc = c; break; }
            }
            NewBlock f = new NewBlock(pc);
            DialogResult r = f.ShowDialog();
            if (r == DialogResult.OK)
            {
                Puzzle.Blocks.Add(f.Block);
                isChanged = true;
            }
            show_blocks();
        }

        private void show_blocks()
        {
            blockImages.Images.Clear();
            foreach (Block b in Puzzle.Blocks) blockImages.Images.Add(b.GetImage());
            listBlock.Items.Clear();
            for (int i = 0; i < blockImages.Images.Count; i++)
            {
                ListViewItem item = new ListViewItem();
                item.ImageIndex = i;
                listBlock.Items.Add(item);
            }
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            for (int i=listBlock.Items.Count-1; i>=0; i--) {
                ListViewItem item = listBlock.Items[i];
                if (item.Checked) Puzzle.Blocks.RemoveAt(item.Index);
                isChanged = true;
            }
            show_blocks();
        }

        private void btnFindSolution_Click(object sender, EventArgs e)
        {
            Puzzle.ClearSolution();
            if (Puzzle.FindSolution2())
            {
                ShowPuzzle s = new ShowPuzzle(this.Puzzle);
                s.ShowDialog();
            }
            else
            {
                MessageBox.Show("No solution found!");
            }
        }

        private void setFileName(string fname)
        {
            FileName = fname;
            this.lblPuzzle.Text = Path.GetFileNameWithoutExtension(FileName) + ": Width-" + Puzzle.PuzzleSize.Width.ToString() +
                "   Height-" + Puzzle.PuzzleSize.Height.ToString() +
                "   Rotation-" + Puzzle.AllowRotation.ToString();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (isChanged && !confirmDiscard()) return;
            NewPuzzle p = new NewPuzzle(5, 5);
            DialogResult r = p.ShowDialog();
            if (r == DialogResult.OK)
            {
                Puzzle = new Puzzle();
                Puzzle.initPuzzle(p.PuzzleWidth, p.PuzzleHeight);
                Puzzle.AllowRotation = p.AllowRotation;
                lblPuzzle.Text = "Width:" + this.Puzzle.PuzzleSize.Width.ToString() +
                    "    Height:" + this.Puzzle.PuzzleSize.Height.ToString() +
                    "    Rotation:" + this.Puzzle.AllowRotation.ToString();
                show_blocks();
                setFileName("NewPuzzle.xml");
                isChanged = true;
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (FileName.Equals("NewPuzzle.xml"))
            {
                saveAsToolStripMenuItem_Click(sender, e);
                return;
            }
            StreamWriter writer = new StreamWriter(FileName, false);
            XmlSerializer x = new XmlSerializer(Puzzle.GetType());
            x.Serialize(writer, Puzzle);
            writer.Close();
            isChanged = false;
        }

        private bool confirmDiscard()
        {
            return (MessageBox.Show("Discard the changes?", "Puzzle changed", MessageBoxButtons.OKCancel) == DialogResult.OK);
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (isChanged && !confirmDiscard()) return;
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "Puzzle Files (*.xml)|*.xml|All Files (*.*)|*.*";
            dlg.CheckFileExists = true;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                StreamReader reader = new StreamReader(dlg.FileName);
                XmlSerializer x = new XmlSerializer(typeof(Puzzle));
                Puzzle = (Puzzle)x.Deserialize(reader);
                reader.Close();
                setFileName(dlg.FileName);
                show_blocks();
                isChanged = false;
            }
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Filter = "Puzzle Files (*.xml)|*.xml|All Files (*.*)|*.*";
            dlg.FileName = FileName;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                StreamWriter writer = new StreamWriter(dlg.FileName, false);
                XmlSerializer x = new XmlSerializer(Puzzle.GetType());
                x.Serialize(writer, Puzzle);
                writer.Close();
                setFileName(dlg.FileName);
                isChanged = false;
            }
        }

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (isChanged && !confirmDiscard()) return;
            this.Close();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Find solution for Block Puzzle\nOliver Li\n2011.11", "Block Puzzle Solution");
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (isChanged && !confirmDiscard()) e.Cancel = true;
        }

        private void changeSizeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NewPuzzle p = new NewPuzzle(Puzzle.PuzzleSize.Width, Puzzle.PuzzleSize.Height);
            DialogResult r = p.ShowDialog();
            if (r == DialogResult.OK)
            {
                Puzzle.initPuzzle(p.PuzzleWidth, p.PuzzleHeight);
                lblPuzzle.Text = "Width:" + this.Puzzle.PuzzleSize.Width.ToString() +
                    "    Height:" + this.Puzzle.PuzzleSize.Height.ToString();
                isChanged = true;
            }
        }

        
    }
}
